<?php

namespace FiloBlu\Storelocator\Model\Plugin\Rma;

use Magento\Rma\Model\Rma as MagentoRma;
use Magento\Store\Model\Store;
use Magento\Sales\Model\Order\Address;
use Magento\Framework\Api\AttributeValueFactory;
use Magento\Rma\Api\RmaAttributesManagementInterface;
use Magento\Framework\App\RequestInterface;
use Psr\Log\LoggerInterface;

class Rma
{
    /**
     * @var \FiloBlu\Storelocator\Helper\Data
     */
    protected $_helper;

    /**
     * @var \FiloBlu\Storelocator\Model\StoresFactory
     */
    protected $_storesFactory;
    /**
     * @var \Magento\Rma\Helper\Data
     */
    protected $_rmaHelper;
    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $_transportBuilder;
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var \Magento\Framework\Translate\Inline\StateInterface
     */
    protected $_inlineTranslation;
    /**
     * @var \Magento\Payment\Helper\Data
     */
    protected $_paymentHelper;
    /**
     * @var Address\Renderer
     */
    protected $_addressRenderer;
    /**
     * @var RequestInterface
     */
    private $request;
    /**
     * @var \Magento\Rma\Model\RmaFactory
     */
    private $rmaFactory;
    /**
     * @var \Magento\Rma\Model\ResourceModel\Rma
     */
    private $rma;
    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * Rma constructor.
     * @param \FiloBlu\Storelocator\Helper\Data $helper
     * @param \FiloBlu\Storelocator\Model\StoresFactory $storesFactory
     * @param \Magento\Rma\Helper\Data $rmaHelper
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation
     * @param \Magento\Payment\Helper\Data $paymentHelper
     * @param Address\Renderer $addressRenderer
     * @param RequestInterface $request
     * @param \Magento\Rma\Model\RmaFactory $rmaFactory
     * @param \Magento\Rma\Model\ResourceModel\Rma $rma
     * @param LoggerInterface $logger
     */
    public function __construct(
        \FiloBlu\Storelocator\Helper\Data $helper,
        \FiloBlu\Storelocator\Model\StoresFactory $storesFactory,
        \Magento\Rma\Helper\Data $rmaHelper,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation,
        \Magento\Payment\Helper\Data $paymentHelper,
        \Magento\Sales\Model\Order\Address\Renderer $addressRenderer,
        RequestInterface $request,
        \Magento\Rma\Model\RmaFactory $rmaFactory,
        \Magento\Rma\Model\ResourceModel\Rma $rma,
        LoggerInterface $logger
    )
    {
        $this->_helper = $helper;
        $this->_storesFactory = $storesFactory;
        $this->_rmaHelper = $rmaHelper;
        $this->_transportBuilder = $transportBuilder;
        $this->_storeManager = $storeManager;
        $this->_inlineTranslation = $inlineTranslation;
        $this->_paymentHelper = $paymentHelper;
        $this->_addressRenderer = $addressRenderer;
        $this->request = $request;
        $this->rmaFactory = $rmaFactory;
        $this->rma = $rma;
        $this->logger = $logger;
    }


    public function afterSave($interceptor, $rma)
    {
        if ($this->_helper->getConfig("carriers/pickupinstore/active")) {
            if ((int)$rma->getData('return_to_store_id') > 0 && $rma->getStatus() == 'pending') {
                try {
                    if ($rma->getData('return_to_store_received_date') == null) {
                        $this->sendMail($rma);
                    }
                } catch (\Exception $ex) {

                }
            }
        }

        /* Send mail to store and to customer if not using standard rma mail flow */
        if (!$this->_helper->getConfig(\FiloBlu\Storelocator\Helper\Data::USE_STANDARD_PICKRMA_MAIL_FLOW)) {
            if ((int)$rma->getData('return_to_store_id') > 0 && $rma->getStatus() == 'authorized') {
                try {
                    if ($rma->getData('return_to_store_received_date') == null) {
                        if ($rma->getOrigData('status') === 'pending') {
                            $this->sendMail($rma);

                            $sendAuthEmail = false;

                            if ($this->request->getParam('hash')) {
                                $sendAuthEmail = true;
                            }

                            $rma->setIsSendAuthEmail($sendAuthEmail);
                        }
                    }
                } catch (\Exception $ex) {

                }
            }
        }

        return $rma;

    }

    protected function sendMail($rma)
    {
        if (!$this->_helper->getConfig("carriers/pickupinstore/pickrma_email_to_store_enabled")) {
            return;
        }

        $store_id = $rma->getData('return_to_store_id');
        $shop = $this->getStoreDetails($store_id);
        if ($shop && $shop->getEmail() != '') {
            $this->_inlineTranslation->suspend();

            $mailTo = $this->_helper->getConfig("carriers/pickupinstore/pickrma_sender_email_to_store_identity");

            $template = $this->_helper->getConfig("carriers/pickupinstore/pickrma_email_to_store_template");
            $mailReceiver = $rma->getReturnToStoreEmail();
            $sendRejectionEmail = $this->_helper->getConfig(\FiloBlu\RmaExtend\Observer\SendRmaRejectedMailToCustomer::EASY_RETURN_SEND_CUSTOMER_RMA_REJECTION_EMAIL);

            $useStandardMailFlow = $this->_helper->getConfig(\FiloBlu\RmaExtend\Helper\Data::EASY_RETURN_USE_STANDARD_MAIL_FLOW);

            if (!$useStandardMailFlow) {
                if ($rma->getStatus() == 'pending') {
                    $mailReceiver = $this->_helper->getConfig(\FiloBlu\RmaExtend\Helper\Data::EASY_RETURN_NON_STANDARD_FLOW_MAIL_SETTINGS);
                    $template = $this->_helper->getConfig(\FiloBlu\RmaExtend\Helper\Data::EASY_RETURN_CUSTOMER_CARE_EMAIL_TEMPLATE);

                    if ($this->_helper->getConfig(\FiloBlu\Storelocator\Helper\Data::PICKRMA_USE_TEST_MAIL_RECIPIENTS)) {
                        if ($this->_helper->getConfig(\FiloBlu\Storelocator\Helper\Data::PICKRMA_CUSTOMERCARE_TEST_MAIL_RECIPIENT)) {
                            $mailReceiver = $this->_helper->getConfig(\FiloBlu\Storelocator\Helper\Data::PICKRMA_CUSTOMERCARE_TEST_MAIL_RECIPIENT);
                        }
                    }
                }

                if ($rma->getStatus() == 'authorized') {
                    if ($this->_helper->getConfig(\FiloBlu\Storelocator\Helper\Data::PICKRMA_USE_TEST_MAIL_RECIPIENTS)) {
                        if ($this->_helper->getConfig(\FiloBlu\Storelocator\Helper\Data::PICKRMA_CUSTOMERCARE_TEST_MAIL_RECIPIENT)) {
                            $mailReceiver = $this->_helper->getConfig(\FiloBlu\Storelocator\Helper\Data::PICKRMA_STORE_TEST_MAIL_RECIPIENT);
                        }
                    }
                }
            }

            try {
                $returnAddress = $this->_rmaHelper->getReturnAddress('html', [], $store_id);
                $order = $rma->getOrder();

                $transport = $this->_transportBuilder
                    ->setTemplateIdentifier($template)
                    ->setTemplateOptions(['area' => \Magento\Framework\App\Area::AREA_FRONTEND, 'store' => $this->_storeManager->getStore()->getStoreId()])
                    ->setTemplateVars(
                        [
                            'rma' => $rma,
                            'order' => $rma->getOrder(),
                            'shop' => $shop,
                            'store' => $rma->getOrder()->getStore(),
                            'billing' => $order->getBillingAddress(),
                            'payment_html' => $this->_paymentHelper->getInfoBlockHtml(
                                $order->getPayment(),
                                $this->_storeManager->getStore()->getId()
                            ),
                            'return_address' => $returnAddress,
                            'item_collection' => $rma->getItemsForDisplay(),
                            'formattedShippingAddress' => $this->_addressRenderer->format(
                                $order->getShippingAddress(),
                                'html'
                            ),
                            'formattedBillingAddress' => $this->_addressRenderer->format(
                                $order->getBillingAddress(),
                                'html'
                            ),
                            'sendRejectionEmail' => $sendRejectionEmail
                        ]
                    )
                    ->setFrom($mailTo)
                    ->addTo($mailReceiver)
                    ->getTransport();
                $transport->sendMessage();

                $this->_inlineTranslation->resume();
            } catch (\Exception $e) {
                $this->_inlineTranslation->resume();
            }
        }
    }

    protected function getStoreDetails($store_id)
    {
        if ((int)$store_id > 0) {
            $shop = $this->_storesFactory->create()->load($store_id);

            return $shop;
        }

        return false;
    }

    /**
     * @param MagentoRma $subject
     * @param $result
     * @return array|null
     */
    public function afterSaveRma(MagentoRma $subject, $result)
    {
        try {
            if (!$result) {
                return $result;
            }
            $data = $this->request->getParams();

            if (isset($data['rma_returntostore_store_id']) && $data['rma_returntostore_store_id'] > 0) {
                $data['return_to_store_id'] = $data['rma_returntostore_store_id'];
                $data['return_to_store_email'] = $data['rma_returntostore_email'];
                $data['return_to_store_hash'] = (md5(time()));
                $rma = $this->rmaFactory->create()->load($result->getId());
                $rma->setData('return_to_store_id', $data['rma_returntostore_store_id'])
                    ->setData('return_to_store_email', $data['rma_returntostore_email'])
                    ->setData('return_to_store_hash', md5(time()))
                ->save();
            }

            return $result;
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            return $result;
        } catch (\Throwable $t) {
            $this->logger->critical($t->getMessage());
            return $result;
        }
    }

}
